@@ -29,26 +29,27 @@ Improvements:
 
 """
 
-import sys
 import datetime
-import six
+import sys
 from optparse import make_option
 
 import django
-from django.db.models import AutoField, BooleanField, FileField, ForeignKey, DateField, DateTimeField
-from django.core.exceptions import ObjectDoesNotExist
-from django.core.management.base import BaseCommand
-
+import six
 # conditional import, force_unicode was renamed in Django 1.5
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ObjectDoesNotExist
+from django.core.management.base import BaseCommand
+from django.db.models import (
+    AutoField, BooleanField, DateField, DateTimeField, FileField, ForeignKey,
+)
+
+from django_extensions.management.utils import signalcommand
 
 try:
     from django.utils.encoding import smart_unicode, force_unicode  # NOQA
 except ImportError:
     from django.utils.encoding import smart_text as smart_unicode, force_text as force_unicode  # NOQA
 
-from django_extensions.management.utils import signalcommand
-
 
 def orm_item_locator(orm_obj):
     """
@@ -559,7 +560,7 @@ class Script(Code):
 # and the script is at ./some_folder/some_script.py
 # you must make sure ./some_folder/__init__.py exists
 # and run  ./manage.py runscript some_folder.some_script
-
+import os, sys
 from django.db import transaction
 
 class BasicImportHelper(object):
