@@ -4,9 +4,12 @@ originally from http://www.djangosnippets.org/snippets/828/ by dnordberg
 import logging
 from optparse import make_option
 
+import django
 from django.conf import settings
-from django.core.management.base import CommandError, BaseCommand
-from six.moves import input
+from django.core.management.base import BaseCommand, CommandError
+from six.moves import configparser, input
+
+from django_extensions.management.utils import signalcommand
 
 
 class Command(BaseCommand):
@@ -35,6 +38,7 @@ class Command(BaseCommand):
     )
     help = "Resets the database for this project."
 
+    @signalcommand
     def handle(self, *args, **options):
         """
         Resets the database for this project.
@@ -52,11 +56,22 @@ class Command(BaseCommand):
             raise CommandError("Unknown database router %s" % router)
 
         engine = dbinfo.get('ENGINE').split('.')[-1]
-        user = options.get('user') or dbinfo.get('USER')
-        password = options.get('password') or dbinfo.get('PASSWORD')
+
+        user = password = database_name = ''
+        if engine == 'mysql':
+            read_default_file = dbinfo.get('OPTIONS', {}).get('read_default_file')
+            if read_default_file:
+                config = configparser.ConfigParser()
+                config.read(read_default_file)
+                user = config.get('client', 'user')
+                password = config.get('client', 'password')
+                database_name = config.get('client', 'database')
+
+        user = options.get('user') or dbinfo.get('USER') or user
+        password = options.get('password') or dbinfo.get('PASSWORD') or password
         owner = options.get('owner') or user
 
-        database_name = options.get('dbname') or dbinfo.get('NAME')
+        database_name = options.get('dbname') or dbinfo.get('NAME') or database_name
         if database_name == '':
             raise CommandError("You need to specify DATABASE_NAME in your Django settings file.")
 
@@ -111,9 +126,9 @@ Type 'yes' to continue, or 'no' to cancel: """ % (database_name,))
             connection.query(create_query)
 
         elif engine in ('postgresql', 'postgresql_psycopg2', 'postgis'):
-            if engine == 'postgresql':
+            if engine == 'postgresql' and django.VERSION < (1, 9):
                 import psycopg as Database  # NOQA
-            elif engine in ('postgresql_psycopg2', 'postgis'):
+            elif engine in ('postgresql', 'postgresql_psycopg2', 'postgis'):
                 import psycopg2 as Database  # NOQA
 
             conn_string = "dbname=template1"
@@ -129,23 +144,27 @@ Type 'yes' to continue, or 'no' to cancel: """ % (database_name,))
             connection = Database.connect(conn_string)
             connection.set_isolation_level(0)  # autocommit false
             cursor = connection.cursor()
-            drop_query = 'DROP DATABASE %s;' % database_name
+            drop_query = "DROP DATABASE \"%s\";" % database_name
             logging.info('Executing... "' + drop_query + '"')
 
             try:
                 cursor.execute(drop_query)
             except Database.ProgrammingError as e:
-                logging.info("Error: %s" % str(e))
+                logging.exception("Error: %s" % str(e))
 
-            create_query = "CREATE DATABASE %s" % database_name
+            create_query = "CREATE DATABASE \"%s\"" % database_name
             if owner:
                 create_query += " WITH OWNER = \"%s\" " % owner
             create_query += " ENCODING = 'UTF8'"
 
-            if engine == 'postgis':
-                # fetch postgis template name if it exists
-                from django.contrib.gis.db.backends.postgis.creation import PostGISCreation
-                postgis_template = PostGISCreation(connection).template_postgis
+            if engine == 'postgis' and django.VERSION < (1, 9):
+                # For PostGIS 1.5, fetch template name if it exists
+                if django.VERSION < (1, 8):
+                    from django.contrib.gis.db.backends.postgis.creation import PostGISCreation
+                    postgis_template = PostGISCreation(connection).template_postgis
+                else:
+                    from django.contrib.gis.db.backends.postgis.base import DatabaseWrapper
+                    postgis_template = DatabaseWrapper(dbinfo).template_postgis
                 if postgis_template is not None:
                     create_query += ' TEMPLATE = %s' % postgis_template
 
