@@ -20,18 +20,18 @@ KNOWN ISSUES:
      positives or false negatives.
 """
 
-import six
 import sys
 from optparse import make_option
 
 import django
+import six
+from django.core.management import CommandError, sql as _sql
 from django.core.management.base import BaseCommand
-from django.core.management import sql as _sql
-from django.core.management import CommandError
 from django.core.management.color import no_style
-from django.db import transaction, connection
-from django.db.models.fields import IntegerField, AutoField
+from django.db import connection, transaction
+from django.db.models.fields import AutoField, IntegerField
 
+from django_extensions.compat import get_app_models
 from django_extensions.management.utils import signalcommand
 
 try:
@@ -153,6 +153,9 @@ class SQLDiff(object):
         self.cursor = connection.cursor()
         self.django_tables = self.get_django_tables(options.get('only_existing', True))
         self.db_tables = self.introspection.get_table_list(self.cursor)
+        if django.VERSION[:2] >= (1, 8):
+            # TODO: We are losing information about tables which are views here
+            self.db_tables = [table_info.name for table_info in self.db_tables]
         self.differences = []
         self.unknown_db_fields = {}
         self.new_db_fields = set()
@@ -275,9 +278,7 @@ class SQLDiff(object):
             kwargs['geography'] = True
 
         if '.' in reverse_type:
-            from django.utils import importlib
-            # TODO: when was importlib added to django.utils ? and do we
-            # need to add backwards compatibility code ?
+            from django_extensions.compat import importlib
             module_path, package_name = reverse_type.rsplit('.', 1)
             module = importlib.import_module(module_path)
             field_db_type = getattr(module, package_name)(**kwargs).db_type(connection=connection)
@@ -311,9 +312,9 @@ class SQLDiff(object):
         for field in all_local_fields(meta):
             if field.unique and meta.managed:
                 attname = field.db_column or field.attname
-                db_field_unique = table_indexes[attname]['unique']
+                db_field_unique = table_indexes.get(attname, {}).get('unique')
                 if not db_field_unique and table_constraints:
-                    db_field_unique = any(constraint['unique'] for contraint_name, constraint in six.iteritems(table_constraints) if attname in constraint['columns'])
+                    db_field_unique = any(constraint['unique'] for contraint_name, constraint in six.iteritems(table_constraints) if [attname] == constraint['columns'])
                 if attname in table_indexes and db_field_unique:
                     continue
                 self.add_difference('unique-missing-in-db', table_name, attname)
@@ -932,7 +933,6 @@ to check/debug ur models compared to the real database tables and columns."""
 
     @signalcommand
     def handle(self, *app_labels, **options):
-        from django.db import models
         from django.conf import settings
 
         engine = None
@@ -949,20 +949,14 @@ because you haven't specified the DATABASE_ENGINE setting.
 Edit your settings file and change DATABASE_ENGINE to something like 'postgresql' or 'mysql'.""")
 
         if options.get('all_applications', False):
-            app_models = models.get_models(include_auto_created=True)
+            app_models = get_app_models()
         else:
             if not app_labels:
                 raise CommandError('Enter at least one appname.')
-            try:
-                app_list = [models.get_app(app_label) for app_label in app_labels]
-            except (models.ImproperlyConfigured, ImportError) as e:
-                raise CommandError("%s. Are you sure your INSTALLED_APPS setting is correct?" % e)
 
-            app_models = []
-            for app in app_list:
-                app_models.extend(models.get_models(app, include_auto_created=True))
+            app_models = get_app_models(app_labels)
 
-        ## remove all models that are not managed by Django
+        # remove all models that are not managed by Django
         #app_models = [model for model in app_models if getattr(model._meta, 'managed', True)]
 
         if not app_models:
