@@ -7,6 +7,33 @@ Based on:
   Adapted to be used with django-extensions
 """
 
+import datetime
+import os
+
+import six
+from django.db import models
+from django.db.models import get_models
+from django.db.models.fields.related import (
+    ForeignKey, ManyToManyField, OneToOneField, RelatedField,
+)
+from django.template import Context, Template, loader
+from django.utils.safestring import mark_safe
+from django.utils.translation import activate as activate_language
+
+try:
+    from django.utils.encoding import force_bytes
+except ImportError:
+    from django.utils.encoding import smart_str as force_bytes
+
+try:
+    from django.db.models.fields.generic import GenericRelation
+    assert GenericRelation
+except ImportError:
+    from django.contrib.contenttypes.generic import GenericRelation
+
+from django_extensions.compat import get_apps
+
+
 __version__ = "1.0"
 __license__ = "Python"
 __author__ = "Bas van Oostveen <v.oostveen@gmail.com>",
@@ -19,30 +46,19 @@ __contributors__ = [
     "Justin Findlay <jfindlay@gmail.com>",
     "Alexander Houben <alexander@houben.ch>",
     "Joern Hees <gitdev@joernhees.de>",
+    "Kevin Cherepski <cherepski@gmail.com>",
+    "Jose Tomas Tocino <theom3ga@gmail.com>"
 ]
 
-import os
-import datetime
-from django.utils.translation import activate as activate_language
-from django.utils.safestring import mark_safe
-from django.template import Context, loader
-from django.db import models
-from django.db.models import get_models
-from django.db.models.fields.related import ForeignKey, OneToOneField, ManyToManyField, RelatedField
-
-try:
-    from django.db.models.fields.generic import GenericRelation
-    assert GenericRelation
-except ImportError:
-    from django.contrib.contenttypes.generic import GenericRelation
-
 
 def parse_file_or_list(arg):
     if not arg:
         return []
-    if not ',' in arg and os.path.isfile(arg):
+    if isinstance(arg, (list, tuple, set)):
+        return arg
+    if ',' not in arg and os.path.isfile(arg):
         return [e.strip() for e in open(arg).readlines()]
-    return arg.split(',')
+    return [e.strip() for e in arg.split(',')]
 
 
 def generate_dot(app_labels, **kwargs):
@@ -72,11 +88,11 @@ def generate_dot(app_labels, **kwargs):
 
     apps = []
     if all_applications:
-        apps = models.get_apps()
+        apps = get_apps()
 
     for app_label in app_labels:
         app = models.get_app(app_label)
-        if not app in apps:
+        if app not in apps:
             apps.append(app)
 
     graphs = []
@@ -120,20 +136,24 @@ def generate_dot(app_labels, **kwargs):
             def consider(model_name):
                 if exclude_models and model_name in exclude_models:
                     return False
+                elif include_models and model_name not in include_models:
+                    return False
                 return not include_models or model_name in include_models
 
             if not consider(appmodel._meta.object_name):
                 continue
 
             if verbose_names and appmodel._meta.verbose_name:
-                model['label'] = appmodel._meta.verbose_name
+                model['label'] = force_bytes(appmodel._meta.verbose_name)
             else:
                 model['label'] = model['name']
 
             # model attributes
             def add_attributes(field):
                 if verbose_names and field.verbose_name:
-                    label = field.verbose_name
+                    label = force_bytes(field.verbose_name)
+                    if label.islower():
+                        label = label.capitalize()
                 else:
                     label = field.name
 
@@ -159,13 +179,13 @@ def generate_dot(app_labels, **kwargs):
 
             # find primary key and print it first, ignoring implicit id if other pk exists
             pk = appmodel._meta.pk
-            if not appmodel._meta.abstract and pk in attributes:
+            if pk and not appmodel._meta.abstract and pk in attributes:
                 add_attributes(pk)
 
             for field in attributes:
                 if skip_field(field):
                     continue
-                if field == pk:
+                if pk and field == pk:
                     continue
                 add_attributes(field)
 
@@ -173,7 +193,7 @@ def generate_dot(app_labels, **kwargs):
                 model['fields'] = sorted(model['fields'], key=lambda field: (not field['primary_key'], not field['relation'], field['label']))
 
             # FIXME: actually many_to_many fields aren't saved in this model's db table, so why should we add an attribute-line for them in the resulting graph?
-            #if appmodel._meta.many_to_many:
+            # if appmodel._meta.many_to_many:
             #    for field in appmodel._meta.many_to_many:
             #        if skip_field(field):
             #            continue
@@ -182,17 +202,25 @@ def generate_dot(app_labels, **kwargs):
             # relations
             def add_relation(field, extras=""):
                 if verbose_names and field.verbose_name:
-                    label = field.verbose_name
+                    label = force_bytes(field.verbose_name)
+                    if label.islower():
+                        label = label.capitalize()
                 else:
                     label = field.name
 
                 # show related field name
                 if hasattr(field, 'related_query_name'):
-                    label += ' (%s)' % field.related_query_name()
+                    related_query_name = field.related_query_name()
+                    if verbose_names and related_query_name.islower():
+                        related_query_name = related_query_name.replace('_', ' ').capitalize()
+                    label += ' (%s)' % related_query_name
 
-                # handle self-relationships
-                if field.rel.to == 'self':
-                    target_model = field.model
+                # handle self-relationships and lazy-relationships
+                if isinstance(field.rel.to, six.string_types):
+                    if field.rel.to == 'self':
+                        target_model = field.model
+                    else:
+                        raise Exception("Lazy relationship for model (%s) must be explicit for field (%s)" % (field.model.__name__, field.name))
                 else:
                     target_model = field.rel.to
 
@@ -269,6 +297,12 @@ def generate_dot(app_labels, **kwargs):
 
     now = datetime.datetime.now()
     t = loader.get_template('django_extensions/graph_models/digraph.dot')
+
+    if not isinstance(t, Template) and not (hasattr(t, 'template') and isinstance(t.template, Template)):
+        raise Exception("Default Django template loader isn't used. "
+                        "This can lead to the incorrect template rendering. "
+                        "Please, check the settings.")
+
     c = Context({
         'created_at': now.strftime("%Y-%m-%d %H:%M"),
         'cli_options': cli_options,
