@@ -1,6 +1,8 @@
-import six
 import traceback
 
+import django
+import six
+
 
 class ObjectImportError(Exception):
     pass
@@ -89,12 +91,21 @@ def import_objects(options, style):
         from django.apps import apps
         from django import setup
     except ImportError:
-        pass
+        from django.db.models.loading import get_models, get_apps
+
+        def get_apps_and_models():
+            for app_mod in get_apps():
+                app_models = get_models(app_mod)
+                yield app_mod, app_models
     else:
         if not apps.ready:
             setup()
 
-    from django.db.models.loading import get_models, get_apps
+        def get_apps_and_models():
+            for app in apps.get_app_configs():
+                if app.models_module:
+                    yield app.models_module, app.get_models()
+
     mongoengine = False
     try:
         from mongoengine.base import _document_registry
@@ -133,8 +144,7 @@ def import_objects(options, style):
             load_models.setdefault(mod.__module__, [])
             load_models[mod.__module__].append(name)
 
-    for app_mod in get_apps():
-        app_models = get_models(app_mod)
+    for app_mod, app_models in get_apps_and_models():
         if not app_models:
             continue
 
@@ -147,13 +157,20 @@ def import_objects(options, style):
             if "%s.%s" % (app_name, mod.__name__) in dont_load:
                 continue
 
-            load_models.setdefault(mod.__module__, [])
-            load_models[mod.__module__].append(mod.__name__)
+            if mod.__module__:
+                # Only add the module to the dict if `__module__` is not empty.
+                load_models.setdefault(mod.__module__, [])
+                load_models[mod.__module__].append(mod.__name__)
 
     if not quiet_load:
         print(style.SQL_TABLE("# Shell Plus Model Imports"))
+
     for app_mod, models in sorted(six.iteritems(load_models)):
-        app_name = app_mod.split('.')[-2]
+        try:
+            app_name = app_mod.split('.')[-2]
+        except IndexError:
+            # Some weird model naming scheme like in Sentry.
+            app_name = app_mod
         app_aliases = model_aliases.get(app_name, {})
         model_labels = []
 
@@ -185,15 +202,19 @@ def import_objects(options, style):
     if getattr(settings, 'SHELL_PLUS_DJANGO_IMPORTS', True):
         if not quiet_load:
             print(style.SQL_TABLE("# Shell Plus Django Imports"))
-        SHELL_PLUS_DJANGO_IMPORTS = (
-            ('django.core.cache', ['cache']),
-            ('django.core.urlresolvers', ['reverse']),
-            ('django.conf', ['settings']),
-            ('django.db', ['transaction']),
-            ('django.db.models', ['Avg', 'Count', 'F', 'Max', 'Min', 'Sum', 'Q']),
-            ('django.utils', ['timezone']),
-        )
-        imports = import_items(SHELL_PLUS_DJANGO_IMPORTS, style, quiet_load=quiet_load)
+        SHELL_PLUS_DJANGO_IMPORTS = {
+            'django.core.cache': ['cache'],
+            'django.core.urlresolvers': ['reverse'],
+            'django.conf': ['settings'],
+            'django.db': ['transaction'],
+            'django.db.models': ['Avg', 'Count', 'F', 'Max', 'Min', 'Sum', 'Q'],
+            'django.utils': ['timezone'],
+        }
+        if django.VERSION[:2] >= (1, 7):
+            SHELL_PLUS_DJANGO_IMPORTS['django.db.models'].append("Prefetch")
+        if django.VERSION[:2] >= (1, 8):
+            SHELL_PLUS_DJANGO_IMPORTS['django.db.models'].extend(["Case", "When"])
+        imports = import_items(SHELL_PLUS_DJANGO_IMPORTS.items(), style, quiet_load=quiet_load)
         for k, v in six.iteritems(imports):
             imported_objects[k] = v
 
